#!/usr/bin/python
#
# Copyright 2016 Canonical Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import json
from charmhelpers.core import hookenv


from charms.reactive import when, when_not, set_state, remove_state

from cinder_contexts import StorwizeSVCSubordinateContext

from charmhelpers.core.hookenv import (
    Hooks,
    UnregisteredHookError,
    config,
    service_name,
    relation_set,
    relation_ids,
    status_set,
    log,
)

from charmhelpers.core.host import (
    service_restart,
)

from charmhelpers.contrib.openstack.utils import (
    os_application_version_set,
)

VERSION_PACKAGE = 'cinder-common'
hooks = Hooks()


def get_storwizesvc_request():

    """
    Function to get the storwize-svc config values
    :param: None
    :returns: string
    """
    volumedriver = config('volume-driver')
    volume_backend_name = config('volume-backend-name')
    san_ip = config('san-ip')
    san_login = config('san-login')
    san_password = config('san-password')
    volpool_name = config('storwize-svc-volpool-name')
    volume_clear = config('volume-clear')

    hookenv.log("Getting config val " + san_login)
    data = {"volume-driver": volumedriver,
            "volume-backend-name": volume_backend_name,
            "san-ip": san_ip,
            "san-login": san_login,
            "san-password": san_password,
            "storwize-svc-volpool-name": volpool_name,
            "volume-clear": volume_clear, }
    if not volume_backend_name or \
       not san_ip or not san_login or \
       not san_password or not volpool_name:
        hookenv.log("Some of the config parameter values are empty, \
Please provide correct values.")
        return ''
    else:
        return data


@when_not('ibm-cinder-storwize-svc.configured')
@when('storage-backend.available')
def configure_storwize_svc(self):

    """
    Function to configure the storwize-svc driver
    :param: None
    :returns: None
    """
    hookenv.log("In configure_storwize_svc fn")
    data = get_storwizesvc_request()
    if not data:
        status_set('blocked', 'Some of the config parameter values are empty, \
Please provide correct values.')
        return 1
    hookenv.log("data is " + str(data))
    for rid in relation_ids('storage-backend'):
        storage_backend_joined(rid, data)
    set_state('ibm-cinder-storwize-svc.configured')


@when('config.changed', 'ibm-cinder-storwize-svc.configured')
def config_changed():

    """
    Function to change states in case any config value changes
    :param: None
    :returns: None
    """

    remove_state('ibm-cinder-storwize-svc.configured')
    remove_state('ibm-cinder-storwize-svc.started')


def storage_backend_joined(rel_id=None, data=None):

    """
    Function to provide the Driver informations to cinder charm
    when storwize-svc and cinder charms joined
    :param: None
    :returns: None
    """

    log('Storwize SVC Volume Driver providing information to cinder charm.')
    relation_set(
        relation_id=rel_id,
        backend_name=service_name(),
        subordinate_configuration=json.dumps
        (StorwizeSVCSubordinateContext()(data)),
        stateless=True,
    )


@when('ibm-cinder-storwize-svc.configured')
@when_not('ibm-cinder-storwize-svc.started')
def restart_cinder_services():

    """
    Function to re-start cinder after config update/change
    :param: None
    :returns: None
    """

    service_restart('cinder-volume')
    set_state('ibm-cinder-storwize-svc.started')
    status_set('active', 'IBM Cinder Storwize SVC: Ready')


@when('storage-backend.changing')
def storage_backend_changed():

    """
    Function to update states when relation changed between
    cinder charm and storwize-svc charm
    :param: None
    :returns: None
    """

    remove_state('ibm-cinder-storwize-svc.configured')
    remove_state('ibm-cinder-storwize-svc.started')


if __name__ == '__main__':
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {} - skipping.'.format(e))
    os_application_version_set(VERSION_PACKAGE)
