#!/usr/bin/env python
#
# Copyright 2016 Canonical Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Basic cinder-storwize-svc functional test.
"""
import amulet
import os
import sys
import yaml


from charmhelpers.contrib.openstack.amulet.deployment import (
    OpenStackAmuletDeployment
)


seconds_to_wait = 100000


class CinderStorwizeSvcBasicDeployment(OpenStackAmuletDeployment):
    """Amulet tests on a basic heat deployment."""

    def __init__(self, series=None, openstack=None, source=None, git=False,
                 stable=True):
        """
        Function to deploy the entire test environment.
        :param: series, openstack, source, git
        :returns: None
        """
        super(CinderStorwizeSvcBasicDeployment,
              self).__init__(series, openstack,
                             source, stable)
        self._add_services()
        self._add_relations()
        self._configure_services()
        self._deploy()

        exclude_services = []

        # Wait for deployment ready msgs, except exclusions
        self._auto_wait_for_status(exclude_services=exclude_services)

        self._check_cinderconffile()
        self.d.sentry.wait(seconds_to_wait)

    def _add_services(self):

        """
        Function: Add the services that we're testing,
                  where ibm-cinder-storwize-svc is
                  local, and the rest of the services
                  are from lp branches that are compatible with
                  the local charm (e.g. stable or next).
        :param: None
        :returns: None
        """
        # Note: ibm-cinder-storwize-svc becomes a cinder subordinate unit.
        this_service = {'name': 'ibm-cinder-storwize-svc'}
        other_services = [
            {'name': 'percona-cluster', 'constraints': {'mem': '3072M'}},
            {'name': 'keystone'},
            {'name': 'rabbitmq-server'},
            {'name': 'cinder'},
        ]
        super(CinderStorwizeSvcBasicDeployment,
              self)._add_services(this_service,
                                  other_services)

    def _add_relations(self):

        """
        Function: Add all of the relations for the services.
        :param: None
        :returns: None
        """
        relations = {
            'cinder:storage-backend':
                'ibm-cinder-storwize-svc:storage-backend',
            'keystone:shared-db': 'percona-cluster:shared-db',
            'cinder:shared-db': 'percona-cluster:shared-db',
            'cinder:identity-service': 'keystone:identity-service',
            'cinder:amqp': 'rabbitmq-server:amqp',
        }
        super(CinderStorwizeSvcBasicDeployment, self)._add_relations(relations)

    def _configure_services(self):

        """
        Function to Configure all of the services.
        :param: None
        :return: None
        """
        keystone_config = {
            'admin-password': 'openstack',
            'admin-token': 'ubuntutesting'
        }
        pxc_config = {
            'dataset-size': '25%',
            'max-connections': 1000,
            'root-password': 'ChangeMe123',
            'sst-password': 'ChangeMe123',
        }
        cinder_config = {
            'block-device': 'None',
            'glance-api-version': '2'
        }
        local_path = os.path.join(os.path.dirname(__file__), 'local.yaml')
        with open(local_path, "r") as fd:
            config = yaml.safe_load(fd)

        san_ip = config.get('ibm-repo').get('storwizesvc_config_sanip')
        if not san_ip:
            message = 'Please provide the san-ip of the storwize svc driver'
            amulet.raise_status(amulet.FAIL, msg=message)
            sys.exit(1)
        san_login = config.get('ibm-repo').get('storwizesvc_config_sanlogin')
        if not san_login:
            message = 'Please provide the san-login of the storwize svc driver'
            amulet.raise_status(amulet.FAIL, msg=message)
            sys.exit(1)
        san_password = config.get('ibm-repo').get('storwizesvc_'
                                                  'config_sanpassword')
        if not san_password:
            message = 'Please provide the san-password for storwize svc driver'
            amulet.raise_status(amulet.FAIL, msg=message)
            sys.exit(1)
        volume_backend_name = config.get('ibm-repo').get('volume_backend_name')
        if not volume_backend_name:
            message = 'Please provide the volume-backend-name \
for storwize svc driver'
            amulet.raise_status(amulet.FAIL, msg=message)
            sys.exit(1)
        storwizesvc_volpool_name = config.get('ibm-repo').get('storwizesvc_'
                                                              'volpool_name')
        if not storwizesvc_volpool_name:
            message = 'Please provide the volpool_name \
for storwize svc driver'
            amulet.raise_status(amulet.FAIL, msg=message)
            sys.exit(1)

        storwize_config = {
            'volume-backend-name': volume_backend_name,
            'san-ip': san_ip,
            'san-login': san_login,
            'san-password': san_password,
            'storwize-svc-volpool-name': storwizesvc_volpool_name,
        }
        configs = {
            'keystone': keystone_config,
            'percona-cluster': pxc_config,
            'cinder': cinder_config,
            'ibm-cinder-storwize-svc': storwize_config,
        }
        super(CinderStorwizeSvcBasicDeployment,
              self)._configure_services(configs)

    def _check_cinderconffile(self):

        """
        Function to check cinder.conf file to confirm
               if storwize-svc driver is configured.
        :param: None
        :return: None
        """
        unit = self.d.sentry['ibm-cinder-storwize-svc'][0]
        output, result_code = unit.run("grep '\[ibm-cinder-storwize-svc\'"
                                       " /etc/cinder/cinder.conf")
        print('Output of grep is %s' % output)

        if result_code != 0:
            message = ('ibm-cinder-storwize-svc is not found in cinder.conf')
            amulet.raise_status(amulet.FAIL, msg=message)
